/*!
  * vue-compose-promise v0.0.1
  * (c) 2019 Eduardo San Martin Morote
  * @license MIT
  */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var compositionApi = require('@vue/composition-api');

function usePromise(options) {
    if (options === void 0) { options = {}; }
    var state = compositionApi.reactive({
        promise: compositionApi.ref(options.promise || null),
        isPending: compositionApi.ref(true),
        data: compositionApi.ref(null),
        error: compositionApi.ref(null),
        isDelayOver: compositionApi.ref(false),
    });
    var timerId = null;
    // reactive to have automatic unwrapping
    var localOptions = compositionApi.reactive({
        pendingDelay: options.pendingDelay == null ? 200 : options.pendingDelay,
    });
    function setupDelay() {
        if (localOptions.pendingDelay > 0) {
            state.isDelayOver = false;
            if (timerId)
                clearTimeout(timerId);
            timerId = setTimeout(function () { return (state.isDelayOver = true); }, localOptions.pendingDelay);
        }
        else {
            state.isDelayOver = true;
        }
    }
    compositionApi.watch(function () { return state.promise; }, function (newPromise) {
        state.isPending = true;
        state.error = null;
        if (!newPromise) {
            state.data = null;
            state.isDelayOver = false;
            if (timerId)
                clearTimeout(timerId);
            timerId = null;
            return;
        }
        setupDelay();
        newPromise
            .then(function (value) {
            // ensure we are dealing with the same promise
            if (state.promise === newPromise) {
                // @ts-ignore seems to be a bug in @vue/composition-api
                state.data = value;
                state.isPending = false;
            }
        })
            .catch(function (err) {
            // ensure we are dealing with the same promise
            if (state.promise === newPromise) {
                state.error = err;
                state.isPending = false;
            }
        });
    });
    return {
        state: state,
        options: localOptions,
        set: function (p) { return (state.promise = p); },
    };
}

function assert(condition, message) {
    if (!condition) {
        // useful when testing
        // console.warn(`[vue-promised] ${message}`)
        throw new Error("[vue-compose-promise] " + message);
    }
}

function getSlotVNode(slots, slotName, data) {
    assert(!!slots[slotName], "No slot \"" + slotName + "\" provided");
    var nodes = slots[slotName](data);
    assert(nodes.length > 0, "Provided scoped slot \"" + slotName + "\" is empty");
    return nodes;
}
var Promised = compositionApi.createComponent({
    props: {
        tag: {
            type: String,
            default: 'span',
        },
        promise: {
            type: Promise,
            required: true,
        },
        pendingDelay: {
            type: Number,
            default: 200,
        },
    },
    setup: function (props, _a) {
        var slots = _a.slots;
        var promised = usePromise({
            pendingDelay: props.pendingDelay,
            promise: props.promise,
        });
        return function () {
            if (slots.combined) {
                var node = slots.combined(promised.state);
                // assert(
                //   (Array.isArray(node) && node.length) || node,
                //   'Provided scoped slot "combined" cannot be empty'
                // )
                // return Array.isArray(node)
                //   ? convertVNodeArray(h, props.tag, node)
                //   : node
                return node;
            }
            if (promised.state.error) {
                return getSlotVNode(slots, 'rejected', promised.state.error);
            }
            if (!promised.state.isPending) {
                return getSlotVNode(slots, 'default', promised.state.data);
            }
            if (!promised.state.isDelayOver)
                return compositionApi.createElement();
            return getSlotVNode(slots, 'pending', promised.state.data);
        };
    },
});

exports.Promised = Promised;
exports.usePromise = usePromise;
